/** @jsx jsx */
import React, { useRef, useState } from 'react';

// eslint-disable-next-line @atlaskit/ui-styling-standard/use-compiled -- Ignored via go/DSP-18766
import { jsx } from '@emotion/react';
import { injectIntl } from 'react-intl-next';
import { useSharedPluginState } from '@atlaskit/editor-common/hooks';
import { getAriaKeyshortcuts, toggleHighlightPalette, tooltip } from '@atlaskit/editor-common/keymaps';
import { highlightMessages as messages } from '@atlaskit/editor-common/messages';
import { expandIconWrapperStyle } from '@atlaskit/editor-common/styles';
import { ColorPalette, getSelectedRowAndColumnFromPalette, highlightColorPalette, highlightColorPaletteWithTokenBorders } from '@atlaskit/editor-common/ui-color';
import { ArrowKeyNavigationType, DropdownContainer as Dropdown, TOOLBAR_BUTTON, ToolbarButton } from '@atlaskit/editor-common/ui-menu';
import { hexToEditorTextBackgroundPaletteColor } from '@atlaskit/editor-palette';
import { akEditorMenuZIndex } from '@atlaskit/editor-shared-styles';
import ExpandIcon from '@atlaskit/icon/glyph/chevron-down';
import { getBooleanFF } from '@atlaskit/platform-feature-flags';
import { Flex } from '@atlaskit/primitives';
import { changeColor, setPalette } from '../commands';
import { EditorHighlightIcon } from './shared/EditorHighlightIcon';
const ToolbarHighlightColor = ({
  popupsMountPoint,
  popupsBoundariesElement,
  popupsScrollableElement,
  isToolbarReducedSpacing,
  disabled,
  pluginInjectionApi,
  intl: {
    formatMessage
  },
  editorView
}) => {
  const toolbarItemRef = useRef(null);
  const [isOpenedByKeyboard, setIsOpenedByKeyboard] = useState(false);
  const {
    highlightState
  } = useSharedPluginState(pluginInjectionApi, ['highlight']);
  const isDropdownOpen = !!(highlightState !== null && highlightState !== void 0 && highlightState.isPaletteOpen);
  const setDropdownOpen = isOpen => {
    if (!(highlightState !== null && highlightState !== void 0 && highlightState.disabled)) {
      const {
        state,
        dispatch
      } = editorView;
      setPalette(pluginInjectionApi, isOpen)(state, dispatch);
    }
  };
  const toggleDropdown = () => setDropdownOpen(!isDropdownOpen);
  const handleClick = () => {
    setIsOpenedByKeyboard(false);
    toggleDropdown();
  };
  const handleKeyDown = event => {
    if (event.key === 'Enter' || event.key === ' ') {
      event.preventDefault();
      setIsOpenedByKeyboard(true);
      toggleDropdown();
    }
  };
  const handleClickOutside = () => {
    if (isDropdownOpen) {
      setDropdownOpen(false);
      setIsOpenedByKeyboard(false);
    }
  };
  const handleEscapeKeydown = () => {
    if (isDropdownOpen) {
      var _toolbarItemRef$curre;
      setDropdownOpen(false);
      setIsOpenedByKeyboard(false);
      toolbarItemRef === null || toolbarItemRef === void 0 ? void 0 : (_toolbarItemRef$curre = toolbarItemRef.current) === null || _toolbarItemRef$curre === void 0 ? void 0 : _toolbarItemRef$curre.focus();
    }
  };
  const handleColorChange = color => {
    var _pluginInjectionApi$c, _pluginInjectionApi$a;
    pluginInjectionApi === null || pluginInjectionApi === void 0 ? void 0 : (_pluginInjectionApi$c = pluginInjectionApi.core) === null || _pluginInjectionApi$c === void 0 ? void 0 : _pluginInjectionApi$c.actions.execute(changeColor(pluginInjectionApi === null || pluginInjectionApi === void 0 ? void 0 : (_pluginInjectionApi$a = pluginInjectionApi.analytics) === null || _pluginInjectionApi$a === void 0 ? void 0 : _pluginInjectionApi$a.actions)({
      color
    }));
    setDropdownOpen(false);
  };

  // Don't render the toolbar option while the plugin is initialising
  if (!highlightState) {
    return null;
  }
  const toolbarButtonLabel = formatMessage(messages.highlight);

  // Get the design token for the  active color (if it exists) to modify the toolbar
  // icon, but show the nice rainbow if none is selected
  const activeColorToken = highlightState.activeColor === null ? null : hexToEditorTextBackgroundPaletteColor(highlightState.activeColor);

  // pixels, used to determine where to horizontally position the dropdown when space is limited
  // this should reflect the width of the dropdown when fully populated with colors, including translations due to layering
  const fitWidth = 242;
  const palette = getBooleanFF('platform.editor.dynamic-palette-borders') ? highlightColorPaletteWithTokenBorders : highlightColorPalette;
  const {
    selectedRowIndex,
    selectedColumnIndex
  } = getSelectedRowAndColumnFromPalette(palette, highlightState.activeColor);
  return jsx(Flex, {
    alignItems: "center"
  }, jsx(Dropdown, {
    mountTo: popupsMountPoint,
    boundariesElement: popupsBoundariesElement,
    scrollableElement: popupsScrollableElement,
    isOpen: isDropdownOpen && !highlightState.disabled,
    handleClickOutside: handleClickOutside,
    handleEscapeKeydown: handleEscapeKeydown,
    zIndex: akEditorMenuZIndex,
    fitWidth: fitWidth,
    closeOnTab: true,
    arrowKeyNavigationProviderOptions: {
      type: ArrowKeyNavigationType.COLOR,
      selectedRowIndex,
      selectedColumnIndex,
      isOpenedByKeyboard,
      isPopupPositioned: true
    },
    trigger: jsx(ToolbarButton, {
      buttonId: TOOLBAR_BUTTON.BACKGROUND_COLOR,
      spacing: isToolbarReducedSpacing ? 'none' : 'default',
      disabled: disabled || highlightState.disabled,
      selected: isDropdownOpen,
      "aria-label": tooltip(toggleHighlightPalette, toolbarButtonLabel),
      "aria-keyshortcuts": getAriaKeyshortcuts(toggleHighlightPalette),
      "aria-expanded": isDropdownOpen,
      "aria-haspopup": true,
      title: tooltip(toggleHighlightPalette, toolbarButtonLabel),
      onClick: handleClick,
      onKeyDown: handleKeyDown,
      ref: toolbarItemRef,
      iconBefore: jsx(Flex, null, jsx(EditorHighlightIcon, {
        selectedColor: activeColorToken,
        disabled: highlightState.disabled
      }), jsx("span", {
        css: expandIconWrapperStyle
      }, jsx(ExpandIcon, {
        label: ""
      })))
    })
  }, jsx("div", {
    "data-testid": "highlight-color-palette"
  }, jsx(ColorPalette, {
    onClick: color => handleColorChange(color),
    selectedColor: highlightState.activeColor,
    paletteOptions: {
      palette,
      hexToPaletteColor: hexToEditorTextBackgroundPaletteColor
    }
  }))));
};
export const ToolbarHighlightColorWithIntl = injectIntl(ToolbarHighlightColor);